#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Wed Apr 18 20:59:22 2018

@author: zerdane
"""

import matplotlib.pylab as plt
import numpy as np
import glob
import os 
from lmfit import Model,minimize, Parameters, Parameter, fit_report
from scipy import exp,sin,sqrt,cos, log10, log, pi
from scipy.special import erf

rep='/home/zerdane/serhane/postdoc_18-20/bernina/EuNiSiGe/proc_data/'

def gaussian(t,a0,t0,sig):
  return (a0/np.sqrt(2*np.pi)/sig*np.exp(-(t-t0)**2/2/sig**2))

def readfile(folder):
    files=sorted(glob.glob(folder+'*.dat'), key=os.path.getmtime)
    return files

def cm2inch(value):
    return value/2.54

def fig_param(FigSize=(14,8),num=None):    
    plt.figure(num=num,figsize=(cm2inch(FigSize[0]),cm2inch(FigSize[1])),dpi=100)
    plt.minorticks_on()
    plt.tick_params(which='minor',direction='in',top='off', right='off',width=0.5,length=3)
    plt.tick_params(which='major',direction='in',top='off', right='off',width=0.5,length=7)
    return

def findScan(files, scan):
    sc=[]
    for i in files:
        bs=os.path.basename(i)[-9:]
        idx=bs.find(str(scan))
        if idx>0:
            print(os.path.basename(i))
            sc=os.path.basename(i)
            break
    return sc

# =============================================================================
# def getData(fname):
#     dat=np.loadtxt(fname).T
#     d={'dist': dat[1], 'lof':dat[2], 'lon':dat[3], 'diff':dat[4],'ref':dat[10]} 
#     return d 
# =============================================================================



def getData(fname):
    dat=np.loadtxt(fname).T
    d={'dist': dat[0], 'lof':dat[1], 'lon':dat[1], 'diff':dat[1],'ref':dat[1]} 
    return d 



# =============================================================================
# 
# def getReflectivity(folder,scI0,scI,deg):
#     files=readfile(folder)
# #    print(files)
#     f0=findScan(files,scan=str(scI0[0]))
#     d=getData(rep+f0)
#     I0=d['lof'].mean() 
#     I=[]
#     for i in range(len(deg)):
#         f=findScan(files,scan=str(scI[i]))
#         d=getData(rep+f)
#         I.append(d['lof'].mean())
#     ratio=I/I0
#     plt.plot(deg,ratio,'o')
#     plt.xlabel('Incident angle /deg')
#     plt.ylabel('R/ R0')
#     return np.array(deg), ratio        
#    
# =============================================================================



def fresnelEq(thr,n2,a=1,n1=1):
    th=np.deg2rad(thr)
    Rs= (n1*cos(th)-n2*sqrt(1-((n1/n2)*sin(th))**2))/(n1*cos(th)+n2*sqrt(1-((n1/n2)*sin(th))**2)) 
    Rp= (n1*sqrt(1-((n1/n2)*sin(th)))-n2*cos(th))/(n1*sqrt(1-((n1/n2)*sin(th)))+n2*cos(th))
#    plt.plot(thr,a*Rs**2,label='Rs')
    plt.plot(thr,a*Rp**2,label='Rp')
    plt.legend()
    return 

def fresnelEqComplex(thr,n,k,a):
    th=np.deg2rad(thr)
    sec=1/cos(th)
    Rp= a*((n-sec)**2+k**2)/(((n+sec)**2+k**2))
#    plt.plot(thr,a*Rs**2,label='Rs')
    plt.plot(thr,Rp,label='Rp')
    plt.legend()
    return

def Rp_Eq(thr,n2,a):
    n1=1
    th=np.deg2rad(thr)
    Rp= (n1*sqrt(1-((n1/n2)*sin(th)))-n2*cos(th))/(n1*sqrt(1-((n1/n2)*sin(th)))+n2*cos(th))
    return a*Rp
    
def RpCom_Eq(thr,k,a,n):
    th=np.deg2rad(thr)

    sec=1/cos(th)
    Rp= a*((n-sec)**2+k**2)/(((n+sec)**2+k**2))
    return Rp

def get_refIndex(thr,ratio):
    mod=Model(RpCom_Eq)
    pars=Parameters() 
    pars.add('k',value=0.02,vary=1, min=0)
    pars.add('a',value=0.1,vary=1, max=1)
    pars.add('n',value=2.1,vary=1)
    out=mod.fit(ratio, pars,thr=thr,fit_kws={'nan_policy': 'omit'})
    print(out.fit_report(show_correl=False))
    plt.plot(thr,ratio,'o')
    plt.plot(thr,out.best_fit,'r-',linewidth=1) 
    
# =============================================================================
# def getplot_shortscan(lm=250,lx=310):
#     fl=[0.2,0.52,1.35,2.7,4.37,6.14,8.75,11.66]
#     Int=[]
#     sc38=dataAna('proc_data/pump_prob/',sc=[38],dis0=7.16,normlof=True, offset=True,fname='0.20 mJ/cm2 (sc38)')
#     Int.append(sc38[1][lm:lx].mean()) 
#     sc23=dataAna('proc_data/pump_prob/',sc=[23],dis0=7.16,normlof=True, offset=True,fname='0.52 mJ/cm2 (sc23)')
#     Int.append(sc23[1][lm:lx].mean())    
#     sc22=dataAna('proc_data/pump_prob/',sc=[22],dis0=7.16,normlof=True, offset=True,fname='1.35 mJ/cm2 (sc22)')
#     Int.append(sc22[1][lm:lx].mean())
#     sc25=dataAna('proc_data/pump_prob/',sc=[25],dis0=7.16,normlof=True, offset=True,fname='2.70 mJ/cm2 (sc25)')
#     Int.append(sc25[1][lm:lx].mean())
#     sc27=dataAna('proc_data/pump_prob/',sc=[27],dis0=7.16,normlof=True, offset=True,fname='4.37 mJ/cm2 (sc27)')
#     Int.append(sc27[1][lm:lx].mean())
#     sc28=dataAna('proc_data/pump_prob/',sc=[28],dis0=7.16,normlof=True, offset=True,fname='6.14 mJ/cm2 (sc28)')
#     Int.append(sc28[1][lm:lx].mean())
#     sc30=dataAna('proc_data/pump_prob/',sc=[30],dis0=7.16,normlof=True, offset=True,fname='8.75 mJ/cm2 (sc30)')
#     Int.append(sc30[1][lm:lx].mean())
#     sc36=dataAna('proc_data/pump_prob/',sc=[36],dis0=7.16,normlof=True, offset=True,fname='11.66 mJ/cm2 (sc36)')
#     Int.append(sc36[1][lm:lx].mean())
#     
#     plt.vlines(x=sc23[0][lm],ymin=-2,ymax=3,colors='r')
#     plt.vlines(x=sc23[0][lx],ymin=-2,ymax=3,colors='r')
# #    sc34=dataAna('proc_data/pump_prob/',sc=[34],dis0=7.16,normlof=True, offset=True,fname='11.66 mJ/cm2 (sc34)')
#     plt.figure(2)
#     fig_param(FigSize=(14,8),num=5)
#     plt.plot(fl,Int,'o-')
#     plt.xlabel('fluence (mJ/cm2)')
#     plt.ylabel('loff-lon/loff')
#     return sc38,sc23,sc22,sc25,sc27,sc28,sc30,sc36
#     
# 
# def getplot_shortscan_average(lm=250,lx=310):
#     fl=[40, 50, 60, 70, 80, 90, 95, 100, 105, 110 ]
#     Int=[]
#     s88=dataAna('proc_data/pump_prob/',sc=[88],dis0=7.16,normlof=True, offset=True, normMin=True,fname='40 K (s88)')
#     Int.append(s88[1][lm:lx].mean()) 
#     s89=dataAna('proc_data/pump_prob/',sc=[89],dis0=7.16,normlof=True, offset=True,normMin=True, fname='50 K (s89)')
#     Int.append(s89[1][lm:lx].mean())    
#     s90=dataAna('proc_data/pump_prob/',sc=[90],dis0=7.16,normlof=True, offset=True,normMin=True, fname='60 K (s90)')
#     Int.append(s90[1][lm:lx].mean())
#     s91=dataAna('proc_data/pump_prob/',sc=[91],dis0=7.16,normlof=True, offset=True,normMin=True,fname='70 K (s91)')
#     Int.append(s91[1][lm:lx].mean())
#     s101=dataAna('proc_data/pump_prob/',sc=[101],dis0=7.16,normlof=True, offset=True,normMin=True,fname='80 K (s101)')
#     Int.append(s101[1][lm:lx].mean())
#     s115=dataAna('proc_data/pump_prob/',sc=[115],dis0=7.16,normlof=True, offset=True,normMin=True,fname='90 K (s115)')
#     Int.append(s115[1][lm:lx].mean())
#     s122=dataAna('proc_data/pump_prob/',sc=[122],dis0=7.16,normlof=True, offset=True,normMin=True,fname='95 K (s122)')
#     Int.append(s122[1][lm:lx].mean())
#     s131=dataAna('proc_data/pump_prob/',sc=[131],dis0=7.16,normlof=True, offset=True,normMin=True,fname='100 k (s131)')
#     Int.append(s131[1][lm:lx].mean())
#     s139=dataAna('proc_data/pump_prob/',sc=[139],dis0=7.16,normlof=True, offset=True,normMin=True,fname='105 K (s139)')
#     Int.append(sc139[1][lm:lx].mean())
#     s150=dataAna('proc_data/pump_prob/',sc=[150],dis0=7.16,normlof=True, offset=True,normMin=True,fname='110 K (s150)')
#     Int.append(s150[1][lm:lx].mean())
#     
#     plt.vlines(x=s88[0][lm],ymin=-2,ymax=3,colors='r')
#     plt.vlines(x=s88[0][lx],ymin=-2,ymax=3,colors='r')
# #    sc34=dataAna('proc_data/pump_prob/',sc=[34],dis0=7.16,normlof=True, offset=True,fname='11.66 mJ/cm2 (sc34)')
#     plt.figure(2)
#     fig_param(FigSize=(14,8),num=5)
#     plt.plot(fl,Int,'o-')
#     plt.xlabel('fluence (mJ/cm2)')
#     plt.ylabel('loff-lon/loff')
#     return s88,s89,s90,s91,s101,sc115,s122,s131,S139,S150
# 
# def getplot_longscan(lm=250,lx=310):
# #    fl=[0.52,6.14,8.75]
# #    Int=[]
#     sc24=dataAna('proc_data/pump_prob/',sc=[24],dis0=7.16,normlof=True, offset=True,fname='0.52 mJ/cm2 (sc24)')
# #    Int.append(sc24[1][lm:lx].mean())    
#     sc29=dataAna('proc_data/pump_prob/',sc=[29],dis0=7.16,normlof=True, offset=True,fname='6.14mJ/cm2 (sc29)')
# #    Int.append(sc29[1][lm:lx].mean())
#     sc33=dataAna('proc_data/pump_prob/',sc=[33],dis0=7.16,normlof=True, offset=True,fname='8.75 mJ/cm2 (sc33)')
# 
# #    plt.vlines(x=sc24[0][lm],ymin=-2,ymax=3,colors='r')
# #    plt.vlines(x=sc24[0][lx],ymin=-2,ymax=3,colors='r')
# ##    sc34=dataAna('proc_data/pump_prob/',sc=[34],dis0=7.16,normlof=True, offset=True,fname='11.66 mJ/cm2 (sc34)')
# #    plt.figure(2)
# #    fig_param(FigSize=(14,8),num=5)
# #    plt.plot(fl,Int,'o-')
# #    plt.xlabel('fluence (mJ/cm2)')
# #    plt.ylabel('loff-lon/loff')
#     return sc24,sc29,sc33
#     
# =============================================================================
    
    
def dataAna(folder,sc=None,dis0=14.2,normlof=None, offset=True,fname=None,normMin=None):
    files=readfile(folder)
    if len(sc)==1:
        ff=findScan(files,scan=str(sc[0]))
        d=getData(folder+ff)
        dis=d['dist']
        diff=d['diff']
#        time=(dis-dis0)/0.15
        time=(dis-dis0)/1

        if normlof:
            diff=d['diff']/d['lof']
            
        if offset:
            diff=diff-diff[1:20].mean()
            fig_param(FigSize=(14,8),num=1)
        if offset and normMin:
            diff=diff/diff.max()
        if fname is None : plt.title(os.path.basename(ff))  
        plt.plot(time, diff*100,'-',linewidth=2,label=fname)
        plt.xlabel('dt(ps)')
        plt.ylabel('loff-lon/loff (x100)')
        plt.legend()            

    if len(sc)!=1:        
        fil=[]
        for j in range(len(sc)):
            ff=findScan(files,scan=str(sc[j]))
            fil.append(ff)
            
        Dist = np.asarray( [getData(rep+f)['dist'] for f in fil] )
        Diff = np.asarray( [getData(rep+f)['diff'] for f in fil] )
        Lon = np.asarray( [getData(rep+f)['lon'] for f in fil] )
        Loff = np.asarray( [getData(rep+f)['lof'] for f in fil] )
        fig_param(FigSize=(14,8),num=2)
        for ii in range(len(sc)):
#            time=(Dist[ii]-dis0)/0.15
            time=(Dist[ii]-dis0)/1
            if normlof:
                out=Diff[ii]/Loff[ii]
            if offset:
                out=out-out[0:3].mean()
            plt.plot(time, out*100,linewidth=2,label='sc_'+str(sc[ii]))
        plt.legend()   
        plt.xlabel('dt(ps)')
        plt.ylabel('loff-lon/loff (x100)')
        fig_param(FigSize=(14,8),num=3)    
        diff=Diff.mean(0)
        lof=Loff.mean(0)
        dis=Dist.mean(0)
#        time=(dis-dis0)/0.15
        time=(dis-dis0)/1
                
        if normlof:
            diff=diff/lof
        if offset:
            diff=diff-diff[1:10].mean()
    
        if fname is None :plt.title('average:'+str(sc),fontsize=8)
        
#        plt.plot(time, ratio,label='ratio')               
        plt.plot(time, diff*100,linewidth=2,label=fname)
#        plt.plot(time, lof,linewidth=2)
#        plt.plot(time, lon,linewidth=2)
#        plt.plot(time, lof/lon,linewidth=2)
        plt.xlabel('dt(ps)')
        plt.ylabel('loff-lon/loff (x100)')
        plt.legend()

    return  time,diff


def data_Ana(folder,sc=None,dis0=14.2,normlof=None, offset=True,fname=None,normMin=None):
    files=readfile(folder)
    if len(sc)==1:
        ff=findScan(files,scan=str(sc[0]))
        d=getData(folder+ff)
        dis=d['dist']
        diff=d['diff']
        Lon=d['lon']
        Loff=d['lof']
#        time=(dis-dis0)/0.15
        time=(dis-dis0)/1

        if normlof:
            diff=d['diff']/d['lof']            
        if offset:
            diff=diff-diff[1:20].mean()
            fig_param(FigSize=(14,8),num=1)
        if offset and normMin:
            diff=diff/diff.max()
        if fname is None : plt.title(os.path.basename(ff))  
        plt.plot(time, diff*100,'-',linewidth=2,label=fname)
        plt.xlabel('dt(ps)')
        plt.ylabel('loff-lon/loff (x100)')
        plt.legend()            
    return  time,diff,dis,Lon, Loff,d['diff'],dis0/1, dis0,ff

    
def save_scan(sc,fname,sample='', temp='', fluence='',comment=''):
    header = ['# \n'
              '# Sample:'+ sample+'\n'
              '# File name: '+sc[-1]+'\n'
              '# Time zero (ps / mm): '+str(sc[-2])+'/'+str(sc[-3])[0:5]+'\n'
              '# Temperature: '+ temp +' K \n'
              '# Fluence: '+ fluence +' mJ/cm2 \n'
              '# Comments: '+ comment+' mJ/cm2 \n'
              '# \n'
              '# Time(ps)      CorrInt          Dist(mm)         Lon        Loff       Lon-LOff \n']    
    fname=fname 
    fname= fname+'.txt' # _EDF_ RIGHT APPROACH
    f = open(fname,'w+')
    f.writelines(item for item in header)
    f.writelines('\t'.join(map(str, [sc[0][i],sc[1][i],sc[2][i],sc[3][i],sc[4][i],sc[5][i]])) + '\n' for i,item in enumerate(sc[0]))
    f.close()
    




def correlation_data(folder,sc=None,dis0=11.3,normlof=False, offset=True,fname=None,plot=1):
    files=readfile(folder)

    ff=findScan(files,scan=str(sc[0]))
    print(ff)
    d=getData(rep+ff)
    dis=d['dist']
    diff=d['diff']
    
    time=(dis-dis0)/0.15
    plt.figure(6)
    plt.plot(dis,diff)
    if normlof:
        diff=d['diff']/d['lof']
    if offset:
        diff=diff-diff[1:40].mean()
    diff=(diff/max(diff))*10
    mod=Model(gaussian)
    pars=Parameters() 
    pars.add('t0',value=0,vary=1)
    pars.add('sig',value=0.1,vary=1, min=0.01)
    pars.add('a0',value=0.5,vary=1)
    out=mod.fit(diff, pars,t=time)
    print(out.fit_report(show_correl=False))
    fwhm=out.values['sig']*2.354
    tt0=out.values['t0']
    amp=out.values['a0']
    mm=tt0*0.15
    if plot:
        fig_param(FigSize=(14,8),num=1)
        plt.title(ff)  
        plt.plot(time-tt0, diff,'o',label=fname)
        plt.plot(time-tt0,out.best_fit,'r-',linewidth=2)
        plt.xlabel('time (ps)')
        plt.xlim(-0.5, 0.5)
        plt.ylabel('Diff')

#        plt.text(0.15, 1200, 't0(ps) ='+str(tt0)[:5], fontsize=7)
#        plt.text(0.15, 1100, 'dist0(mm) ='+str(mm)[:5], fontsize=7)
        plt.text(0.15, 1000, 'FWHM(fs) ='+str(fwhm*1000)[:5], fontsize=10)
        plt.legend()
    return  tt0, fwhm*1000, amp 
    
def getParam(scans,pos):
    amp=[]
    fwhm=[]
    tt0=[]
    for i in scans:
        sc=[i]
        data=correlation_data(folder='proc_data/',sc=sc,dis0=14.7,norm=1,plot=0)
        tt0.append(data[0])
        fwhm.append(data[1])
        amp.append(data[2])
    fig_param(FigSize=(14,8),num=2)
    plt.plot(pos,fwhm,'o')
    plt.ylabel('FWHM(fs)')
    plt.xlabel('Compressor stage position')
    return
    
def do_FFT_Time_Freq(t,tr,a0=100,sig=0.2):
    
    td_fft=np.zeros((len(t)))
    for i in range(len(t)):
        spec=tr*gaussian(t,a0=a0,t0=t[i],sig=sig)
        amp_fft=do_fft(t,spec,plot=False)[1]
        td_fft=np.dstack((td_fft,amp_fft))
    freq=do_fft(t,spec,plot=False)[0]
    td_fft=td_fft
    fig_param(FigSize=(12,8),num=3)    
    x=t[0:1500]; y=freq[0:60]; z=td_fft[0][0:60,0:1500]
#    plt.pcolor(t[10:600],freq[4:25],td_fft[0][4:25,10:600],vmin=-5,vmax=5)

    plt.imshow(z, cmap='jet',
           extent=[x.min(), x.max(), y.min(), y.max()],
           interpolation='spline36', origin='lower',clim=(z.min(),z.max()),aspect=2)
    plt.ylabel('Frequency / THz')
    plt.xlabel('dt / ps')
 
    plt.axis([-1,5,0.1,10])
    plt.tight_layout()
    return t,freq,td_fft

def cosfun(x,a,a0,tosc,phi):
    return a+(a0*np.cos(x*tosc+phi))

def getwaveplateCalib(wp,pw,offset=1,fit=1, Ampp='1.1 mW',pp_sz=[380,290]):
    pw=np.array(pw)
    fl=pw/1000/((pp_sz[0]*pp_sz[1])*1e-8)
    if offset: pw=pw-min(pw)
    fig_param(FigSize=(14,8),num=5)
    plt.plot(wp,fl,'o')
    plt.title('P0='+Ampp+ ', 1kHz, Pump_sz='+str(pp_sz)+'um')
    plt.xlabel('waveplate angle')
    plt.ylabel('fluence mJ/cm2')
    if fit:
        mod=Model(cosfun)
        pars=mod.make_params()
        pars.add('a',value=8 ,  vary =1)
        pars.add('tosc',value=5 ,  vary =1)
        pars.add('phi',value=0 ,  vary =1)
        pars.add('a0',value=2 ,  vary =1)
        out=mod.fit(fl, pars,x=wp,fit_kws={'nan_policy': 'omit'})
        print(out.fit_report(show_correl=False))
        plt.plot(wp,out.best_fit,'r-',linewidth=2)

    return

def do_fft(t,tr,lm=0,lx=None,plot=None,fname=None):
    tr=tr[lm:lx]
    t=t[lm:lx]
    N=t.size
    T = t[N-1]-t[0]
    tfd=np.fft.fft(tr)/N
    ampl =np.absolute(tfd)
    freq=np.arange(N)*1.0/T
    if plot:        
        fig_param(FigSize=(14,10),num=5)
        plt.plot(freq[1:300],ampl[1:300]*100,'-o',linewidth=2,label=fname)
        plt.xlabel("Frequency / THz")
        plt.ylabel("Amplitude x 100")
        plt.tight_layout()
        plt.grid()
        plt.legend(frameon=True)
    return freq,ampl

def Gaus_Cov_StepFunc(x,w):
    return 0.5*(1-erf(-x/(w*sqrt(2))))

def Gaus_Cov_ExpD_StepF(x,w,tau):
    return 0.5*exp(-(2*tau*x-w**2)/(2*tau**2))*(1-erf((-tau*x+w**2)/(sqrt(2)*tau*w)))

           
def Gaus_Cov_ExpC_StepF(x,w,tau):
    return Gaus_Cov_StepFunc(x,w)-Gaus_Cov_ExpD_StepF(x,w,tau)

def GED_GEC_4(t,Amp1,tau1,Amp2,tau2,Amp3,tau3,Amp4, tau4,bg,cts, w=0.01):
  if (w <= 0.001): w=0.01
  GSF = Gaus_Cov_StepFunc(t,w)
  GED1= Gaus_Cov_ExpD_StepF(t,w,tau1)
  GED2= Gaus_Cov_ExpD_StepF(t,w,tau2)
  GED3= Gaus_Cov_ExpD_StepF(t,w,tau3)
  GED4= Gaus_Cov_ExpD_StepF(t,w,tau4)
  
  GED=Amp1*GED1+Amp2*GED2+Amp3*GED3+Amp4*GED4  
  GEC=Amp1*(GSF-GED1)+Amp2*(GSF-GED2)+Amp3*(GSF-GED3)+Amp4*(GSF-GED4)   
  GECDD=Amp1*(GSF-GED1)+Amp2*GED2+Amp3*GED3  
  GECDDD=Amp1*(GSF-GED1)+Amp2*GED2+Amp3*GED3+Amp4*GED4
  GECCD=Amp1*(GSF-GED1)+Amp2*(GSF-GED2)+Amp3*GED3 

  GECTS=Amp2*GED2+  cts * GSF
  
  return bg+GED,bg+GEC,bg+GECDD,bg+GECDDD,bg+GECCD, bg+GECTS



def Model_4exp_D(t,t0,Amp1,tau1,Amp2,tau2,Amp3,tau3,Amp4,tau4,bg,cts,w):
    tt=t-t0
    GED,GEC,GECDD,GECDDD,GECCD,GECTS=GED_GEC_4(tt,Amp1,tau1,Amp2,tau2,Amp3,tau3,Amp4,tau4,bg,cts,w)
    Fun=GED
    return Fun
    
def Model_4exp_C(t,t0,Amp1,tau1,Amp2,tau2,Amp3,tau3,Amp4,tau4,bg,cts,w):
    tt=t-t0
    GED,GEC,GECDD,GECDDD,GECCD,GECTS=GED_GEC_4(tt,Amp1,tau1,Amp2,tau2,Amp3,tau3,Amp4,tau4,bg,cts,w)
    Fun=GEC
    return Fun  

   
def Model_4exp_CDD(t,t0,Amp1,tau1,Amp2,tau2,Amp3,tau3,Amp4,tau4,bg,cts,w):
    tt=t-t0
    GED,GEC,GECDD,GECDDD,GECCD,GECTS=GED_GEC_4(tt,Amp1,tau1,Amp2,tau2,Amp3,tau3,Amp4,tau4,bg,cts,w)
    Fun=GECDD
    return Fun

def Model_4exp_CCD(t,t0,Amp1,tau1,Amp2,tau2,Amp3,tau3,Amp4,tau4,bg,cts,w):
    tt=t-t0
    GED,GEC,GECDD,GECDDD,GECCD,GECTS=GED_GEC_4(tt,Amp1,tau1,Amp2,tau2,Amp3,tau3,Amp4,tau4,bg,cts,w)
    Fun=GECCD
    return Fun

def Model_4exp_CTS(t,t0,Amp1,tau1,Amp2,tau2,Amp3,tau3,Amp4,tau4,bg,cts,w):
    tt=t-t0
    GED,GEC,GECDD,GECDDD,GECCD,GECTS=GED_GEC_4(tt,Amp1,tau1,Amp2,tau2,Amp3,tau3,Amp4,tau4,bg,cts,w)
    Fun=GECTS
    return Fun

def Model_4exp_CDDD(t,t0,Amp1,tau1,Amp2,tau2,Amp3,tau3,Amp4,tau4,cts,w):
    tt=t-t0
    GED,GEC,GECDD,GECDDD,GECCD,GECTS=GED_GEC_4(tt,Amp1,tau1,Amp2,tau2,Amp3,tau3,Amp4, tau4,cts,w)
    Fun=GECDDD
    return Fun


def exp_model(t,t0,amp1,tau1, amp2,tau2,bg):
    tt=t-t0
    return bg+ (amp1 * np.exp(-(tt)/tau1) + amp2 * np.exp(-(tt)/tau2))

def Fit_Model_4exp(ti,tr,Tmin=0,Tmax=-1,pars=None,ExpType='D',osci=0,fname=None,plot=1):
    plt.clf()
    pars=parsC()

    ti=ti[Tmin:Tmax]
    tr=tr[Tmin:Tmax]  
    fig_param(FigSize=(14,10),num=1)  
    if ExpType=='C':
        mod=Model(Model_4exp_C)
    elif ExpType=='D':
        mod=Model(Model_4exp_D)
    elif ExpType=='CDD':
        mod=Model(Model_4exp_CDD)
    elif ExpType=='CDDD':
        mod=Model(Model_4exp_CDDD)
    elif ExpType=='CCD':
        mod=Model(Model_4exp_CCD)
     
    print(len(ti),len(tr))    
 #   out=mod.fit(tr, pars,t=ti,fit_kws={'nan_policy': 'omit'})
    out=mod.fit(tr, pars,t=ti, nan_policy='omit')
    print(out.fit_report(show_correl=False))
    ti=ti-out.values['t0']
    rest=tr-out.best_fit
    plt.plot(ti,rest,'b-',linewidth=2) 
    plt.plot(ti,tr,'k-',linewidth=2)
    plt.plot(ti,out.best_fit,'r-',linewidth=2)
    
###    
##    
#    np.savetxt('best_fit_Eu_Femtolab_1.35mJ', np.array(out.best_fit).T)
#    print(out.best_fit)
#####    
#    plt.figure()
#    t = np.arange(-0.01, 5, 0.01)
#    plt.plot(t,  0.0155*np.exp(-t*0.14568)  -0.01022*np.exp(-t*0.50440), 'g--')
#    plt.show()
#    
        
#        plt.figure()
#        t = np.arange(-0.01, 5, 0.01)
#        plt.plot(t,   -0.01022*np.exp(-t*0.50440), 'g--')
#        plt.show()
#        
#        
#        plt.figure()
#        t = np.arange(-0.01, 5, 0.01)
#        plt.plot(t,  0.0155*np.exp(-t*0.14568) , 'r--')
#        plt.show()
#
###############    
#
#    np.savetxt('Best_Fit_3exp_xray_90K__#129_2.5mj', np.array(out.best_fit).T)
#    print(out.best_fit)
#####
#    return ti,rest


        
def parsC():
    parsC= Parameters()  
    parsC.add('t0',value= -0.09562778,  vary =1,  min=-0.8,max=0.8 )
    
    parsC.add('tau1',value= 0.045,  vary =1, min=0.035, max=0.11 )  #p(7)  - t3  dont change   min=0.035, max=0.11
    parsC.add('Amp1',value= -0.02,  vary = 1, max = -0.0) #p(6) - A3 plot              min=-0.1, max = -0.0

    parsC.add('tau2',value= 0.40838025,  vary =1,  min=0)  # tau1 plot              min=0, max = 1.5
    parsC.add('Amp2',value= 0.06329524,  vary =1, min=0.00)  #p2, A1                min=0.00, max=0.12

    parsC.add('tau3',value= 0.231292,   vary =1, min=0)  #p(5), tau   min=0
    parsC.add('Amp3',value= -0.00441755,  vary =1, max=0) #p(4), Ampl   max=0

  
    
       
    
    parsC.add('bg',value=  0.0,   vary = 1)

    parsC.add('tau4',value=1,  vary =0, min=0.)
    
    parsC.add('Amp4',value=0.0,  vary =0)
    
    parsC.add('w',value=0.04,  vary =0, min=0.01,max=0.09)

    parsC.add('cts',value=  0,  vary = 0)
    
    

    parsC.add('cts',value=  0,  vary = 0)
    parsC.add('A0',value=0e-5,       vary=0, max=0)
    parsC.add('A1',value=-0e-5,       vary=0)
    parsC.add('A2',value=0e-5,       vary=0)
    parsC.add('A3',value=-0e-5,       vary=0)
    parsC.add('A4',value=-0e-5,       vary=0)

    return parsC
