#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Wed Jul  4 17:17:37 2018

@author: esposito_v
"""

import os
import sys
import glob
import pandas as pd
import h5py
import numpy as np
import dask
from pathlib import Path
from dask.base import tokenize
from dask import array

import dataAna_utils as du

sys.path.append('/gpfs/photonics/swissfel/res/bernina-staff/p17295/cnf/src/pythonpath/')
from escape.parse.swissfel import parseScanEco_v01


class experiment(object):    
    def __init__(self, inputs = True):
        if inputs == True:
            print('Welcome to your new experiment. \nPlease enter a mandatory settings beore proceeding.')
            self.experiment_type = input('Enter the type of experiment:')
            self.save_prefix = input('Choose a prefix for the files that will be saved:')
            self.config_folder = input('Scan info folder:')
            self.save_dir = input('Save directory:')
            
        else:
            self.experiment_type = []
            self.pgroup_dir = []
            self.save_prefix = []
            self.config_folder = []
            self.save_dir = []
            self.gainJF = []
        
#        if not os.path.exists(self.save_dir):
#            os.mkdir(self.save_dir)
#            subprocess.call(['chmod', '-R', 'g+rw', self.save_dir])
            
    def __call__(self):
        print('Experiment settings: \n')
        members = [attr for attr in dir(self) if not callable(getattr(self, attr)) and not attr.startswith("__")]
        for member in members:
            if member is 'daq_labels':
                continue
            else:
                print(member + ' : ' +  self.__dict__[member])
        print('\n\n')

        
        
        

class sf_scan_ana(object): 
    def __init__(self, experiment, pedestals=None):
        
        """ Config given from experiment class """
        try:
            self.config_folder = experiment.config_folder
            self.experiment = experiment.experiment_type
            self.save_prefix = experiment.save_prefix
            self.save_dir = experiment.save_dir
            self.gainJF = experiment.gainJF
        except:
            print('\n\nPlease setup experiment.')
        
        self.fnames = []
        self.scan_info = []
        
        if pedestals is None:
            pfolder = Path(experiment.pgroup_dir)
            pfolder = pfolder / Path('res/JF_pedestal')
            self.pedestals = find_last_pedestal(folder=pfolder)
            print('\nNo pedestals given, last pedestal will be used: %s\n' %self.pedestal)
        else:
            self.pedestals = pedestals
        
        self.data = pd.DataFrame()
        
        """ Images diagnostics """
        self.JFsum = dict()
        self.histo = dict()
        self.histo_filt = dict()
        
        """ Other """
        self.isana = []
        self.warning = []
#        self.force_ana = 0
        
        print('Have a good (beam)time!\n\n')
        
        
        
    def add_file(self, newfiles):
        "add files"
            
        if isinstance(newfiles, int):
            newfiles = str(newfiles)
            if len(newfiles)==1:
                newfiles = '00'+newfiles
            elif len(newfiles)==2:
                newfiles = '0'+newfiles
            file = glob.glob(self.config_folder + '*'+'run'+newfiles+'*')
            newfiles = os.path.split(file[0])[-1]
        
        elif isinstance(newfiles, list):
            if isinstance(newfiles[0], int):
                files = []
                for newfile in newfiles:
                    newfile = str(newfile)
                    if len(newfile)==1:
                        newfile = '00'+newfile
                    elif len(newfile)==2:
                        newfile = '0'+newfile
#                    file = glob.glob(self.config_folder + '*'+ 'run'+newfile+'*')
                    file = glob.glob(self.config_folder + '*'+ 'scan'+newfile+'*')
                    file = os.path.split(file[0])[-1]
                    files.append(file)
                newfiles = files
                
        if isinstance(newfiles, str):
            newfiles = [newfiles, ]
            newfiles = sorted(newfiles)
                
                
        for fname in newfiles:
            isempty = not self.scan_info
                
            if fname in self.fnames:
                print('File %s already exists' %fname)
                continue
            
            fname_full = self.config_folder+fname
            scan_info = du.readJson(fname_full)
            
            if isempty:
                self.scan_motor = scan_info['scan_parameters']
            else:
                if not (scan_info['scan_parameters']['Id'] == self.scan_motor['Id']):
                    print('File %s has a different scan_motor than expected.' %fname)
                    continue
                               
            self.scan_info.append(scan_info)
            self.fnames.append(fname)
            self.isana.append(False)
            
            print('Scan info for file %s uploaded.' %fname)
        print('\n\nHave fun analyzing!\n\n')

    
    
    def remove_file(self, files):
        if isinstance(files, str):
            files = [files, ]
            files = sorted(files)
            
        for file in files:
            self.fnames.remove(file)



class detector:
    def __init__(self, gainFile=None, pedestalFile=None, filters=None, ana_fun=None, ana_args=None):
        if (not gainFile is None) and (not pedestalFile is None):
            self.gains, self.pedestals = self.load_gain_pedestal(gainFile, pedestalFile)
        self.filters = filters
        self.ana_fun = ana_fun
        self.ana_args = ana_args
        
    def load_gain_pedestal(self, gainFile, pedestalFile):
        assert isinstance(gainFile, str), 'gain file invalid'
        assert isinstance(pedestalFile, str), 'pedestal file invalid'
        gains = readF(gainFile,"gains")
        pedestals, pixel_mask = readF(pedestalFile,["gains","pixel_mask"])
        return gains, pedestals



class JFdata:
    def __init__(self, raw, detector=None):
        self.raw = raw
        if (not detector is None):
            self.detector = detector
            self.corrected = self.apply_gain_pede_np(detector.gains, detector.pedestals)
    
    def apply_gain_pede_np(self, G, P, pixel_mask=None):
        image = self.raw
        
        mask = int('0b' + 14 * '1', 2)
        mask2 = int('0b' + 2 * '1', 2)
    
        gain_mask = np.bitwise_and(np.right_shift(image, 14), mask2)
        data = np.bitwise_and(image, mask)
    
        m1 = gain_mask == 0
        m2 = gain_mask == 1
        m3 = gain_mask >= 2
        if G is not None:
            g = m1*G[0] + m2*G[1] + m3*G[2]
        else:
            g = np.ones(data.shape, dtype=np.float32)
        if P is not None:
            p = m1*P[0] + m2*P[1] + m3*P[2]
        else:
            p = np.zeros(data.shape, dtype=np.float32)
        if pixel_mask is not None:
            dv,mv = np.broadcast_arrays(data,pixel_mask)
            data[mv] = np.nan
        res = np.divide(data - p, g)
        return res
    
    def apply_filter(self, hist_range=None):
        return filterImage(self.corrected, self.detector.filters, hist_range=hist_range)





def find_last_pedestal(folder):
    files=list(glob.glob( os.path.join(folder,"pedestal_*")) )
    files.sort()
    return files[-1]

def getHist(imgs, bins=np.arange(-2,20,0.05)):
    """ get intensity histogram from a stack of imgs """
    
    if isinstance(imgs, dask.array.Array):
            H = array.histogram(imgs[dask.array.isfinite(imgs)],bins=bins)[0]
            return H,bins
    else:
        H = np.histogram(imgs[np.isfinite(imgs)],bins)[0]
        return np.asarray(H),bins


def load_gain_pedestal(sf_ana):
    if isinstance(sf_ana.gainJF, dict):
        gains = dict()
        for key in sf_ana.gainJF.keys():
            gains[key] = readF(sf_ana.gainJF[key],"gains")
    else:
        gains = readF(sf_ana.gainJF,"gains")
        
    if isinstance(sf_ana.pedestals, dict):
        pedestals = dict()
        for key in sf_ana.pedestals.keys():
            pedestals[key], pixel_mask = readF(sf_ana.pedestals[key],["gains","pixel_mask"])
    else:
        pedestals, pixel_mask = readF(sf_ana.pedestals,["gains","pixel_mask"])
    return gains, pedestals
    
    
def readF(fname,what):
    """ read given field in a h5 file """
    if not isinstance(what,(list,tuple)): what = (what,)
    f = h5py.File(fname,"r")
    ret = [f[w][:] for w in what]
    if len(ret) == 1: ret=ret[0]
    f.close()
    return ret


def correct_stripeJF(imgin):
    imgout=np.zeros((86,(1024*3+18)),dtype=int)
#     256 not divisible by 3, so we round up
#     18 since we have 6 more pixels in H per gap

#     first we fill the normal pixels, the gap ones will be overwritten later
    for yin in range(256) :
          for xin in range(1024) :
              ichip=int(xin/256)
              xout=(ichip*774)+(xin%256)*3+yin%3
              ## 774 is the chip period, 256*3+6
              yout=int(yin/3)
              imgout[yout,xout]=imgin[yin,xin]
    
    
    # now the gap pixels...
    for igap in range(3) :
          for yin in range(256):
              yout=int(yin/6)*2
              #first the left side of gap
              xin=igap*64+63
              xout=igap*774+765+yin%6
              imgout[yout,xout]=imgin[yin,xin]
              imgout[yout+1,xout]=imgin[yin,xin]
              #then the right side is mirrored
              xin=igap*64+63+1
              xout=igap*774+765+11-yin%6
              imgout[yout,xout]=imgin[yin,xin]
              imgout[yout+1,xout]=imgin[yin,xin]

#              if we want a proper normalization (the area of those pixels is double, so they see 2x the signal)
#              imgout[yout,xout]=imgout[yout,xout]/2 
    return imgout


#def correct_stripeJF_dask(imgs_in):
#    
#    chunks = (imgs_in.chunk[0], 86, 3090)
#    
#    name = 'strip_correct-' + tokenize(imgs_in)  # unique identifier
#    
#    dsk = {(name, xout, yout): ()
#            }


def filterImage(imgs, filters, hist_range = None):
    """
    Apply filters on pixels.
    INPUT:
        img: input images: single or stack of images
        filters: intensity interval to be retained: [[int1_min, int1_max], [int1_min, int1_max], ...]
        intensity histogram before and after the filtering can be obtained by inputing an histo_range
        ATTENTION: the histogram part does not work with dask.array.array inputs
    """
    
    if not hist_range is None:
        hist_before,E = getHist(imgs, bins=hist_range)
#        hist_before = np.sum(hist_before,0)
        
    idx = np.zeros(imgs.shape)
    idx = idx.astype(bool)
    for filt in filters:
        temp = (imgs>filt[0]) & (imgs<filt[1])
        temp = temp.astype(bool)
        idx = np.logical_or(idx, temp)
    imgs[~idx] = 0
    
    if not hist_range is None:
        hist_after,E = getHist(imgs, bins=hist_range)
#        hist_after = np.sum(hist_after,0)
        
        return imgs, [hist_before, hist_after]
    else:
        return imgs
    

#def scan_pos_full(data, motor_pos):
    
        
        





















